@@ -40,6 +40,7 @@
 #include "kis_outline_generator.h"
 #include <kis_iterator_ng.h>
 #include "kis_lod_transform.h"
+#include "kundo2command.h"
 
 
 struct Q_DECL_HIDDEN KisPixelSelection::Private {
@@ -72,8 +73,8 @@ KisPixelSelection::KisPixelSelection(KisDefaultBoundsBaseSP defaultBounds, KisSe
     m_d->parentSelection = parentSelection;
 }
 
-KisPixelSelection::KisPixelSelection(const KisPixelSelection& rhs)
-        : KisPaintDevice(rhs)
+KisPixelSelection::KisPixelSelection(const KisPixelSelection& rhs, KritaUtils::DeviceCopyMode copyMode)
+        : KisPaintDevice(rhs, copyMode)
         , KisSelectionComponent(rhs)
         , m_d(new Private)
 {
@@ -86,6 +87,20 @@ KisPixelSelection::KisPixelSelection(const KisPixelSelection& rhs)
     m_d->thumbnailImageTransform = rhs.m_d->thumbnailImageTransform;
 }
 
+KisPixelSelection::KisPixelSelection(const KisPaintDeviceSP copySource, KritaUtils::DeviceCopyMode copyMode, KisSelectionWSP parentSelection)
+    : KisPaintDevice(0, KoColorSpaceRegistry::instance()->alpha8(), copySource->defaultBounds())
+    , m_d(new Private)
+{
+    KisPaintDeviceSP tmpDevice = new KisPaintDevice(*copySource, copyMode, 0);
+    tmpDevice->convertTo(this->colorSpace());
+
+    this->makeFullCopyFrom(*tmpDevice, copyMode, 0);
+
+    m_d->parentSelection = parentSelection;
+    m_d->outlineCacheValid = true;
+    m_d->invalidateThumbnailImage();
+}
+
 KisSelectionComponent* KisPixelSelection::clone(KisSelection*)
 {
     return new KisPixelSelection(*this);
@@ -150,6 +165,9 @@ void KisPixelSelection::applySelection(KisPixelSelectionSP selection, SelectionA
     case SELECTION_INTERSECT:
         intersectSelection(selection);
         break;
+    case SELECTION_SYMMETRICDIFFERENCE:
+        symmetricdifferenceSelection(selection);
+        break;
     default:
         break;
     }
@@ -255,6 +273,32 @@ void KisPixelSelection::intersectSelection(KisPixelSelectionSP selection)
     m_d->invalidateThumbnailImage();
 }
 
+void KisPixelSelection::symmetricdifferenceSelection(KisPixelSelectionSP selection)
+{
+    QRect r = selection->selectedRect().united(selectedRect());
+    if (r.isEmpty()) return;
+
+    KisHLineIteratorSP dst = createHLineIteratorNG(r.x(), r.y(), r.width());
+    KisHLineConstIteratorSP src = selection->createHLineConstIteratorNG(r.x(), r.y(), r.width());
+    for (int i = 0; i < r.height(); ++i) {
+
+        do {
+            *dst->rawData() = abs(*dst->rawData() - *src->oldRawData());
+        }  while (src->nextPixel() && dst->nextPixel());
+
+        dst->nextRow();
+        src->nextRow();
+    }
+    
+    m_d->outlineCacheValid &= selection->outlineCacheValid();
+
+    if (m_d->outlineCacheValid) {
+       m_d->outlineCache = (m_d->outlineCache | selection->outlineCache()) - (m_d->outlineCache & selection->outlineCache());
+    }
+
+    m_d->invalidateThumbnailImage();
+}
+
 void KisPixelSelection::clear(const QRect & r)
 {
     if (*defaultPixel().data() != MIN_SELECTED) {
@@ -359,7 +403,7 @@ QVector<QPolygon> KisPixelSelection::outline() const
     QRect selectionExtent = selectedExactRect();
 
     /**
-     * When the default pixel is not fully transarent, the
+     * When the default pixel is not fully transparent, the
      * exactBounds() return extent of the device instead. To make this
      * value sane we should limit the calculated area by the bounds of
      * the image.
@@ -384,7 +428,7 @@ QVector<QPolygon> KisPixelSelection::outline() const
         delete[] buffer;
         return paths;
     }
-    catch(std::bad_alloc) {
+    catch(const std::bad_alloc&) {
         // Allocating so much memory failed, so we fall through to the slow option.
         warnKrita << "KisPixelSelection::outline ran out of memory allocating" << width << "*" << height << "bytes.";
     }
