@@ -40,6 +40,7 @@
 #include <ksqueezedtextlabel.h>
 #include <kpluginfactory.h>
 
+#include <KisUsageLogger.h>
 #include <KoFileDialog.h>
 #include <kis_icon_utils.h>
 #include <KoDialog.h>
@@ -48,7 +49,6 @@
 #include <KisMimeDatabase.h>
 #include <kis_config_widget.h>
 #include <kis_debug.h>
-#include <KisMimeDatabase.h>
 #include <KisPreExportChecker.h>
 #include <KisPart.h>
 #include "kis_config.h"
@@ -62,6 +62,7 @@
 #include "kis_popup_button.h"
 #include <kis_iterator_ng.h>
 #include "kis_async_action_feedback.h"
+#include "KisReferenceImagesLayer.h"
 
 // static cache for import and export mimetypes
 QStringList KisImportExportManager::m_importMimeTypes;
@@ -81,13 +82,13 @@ struct KisImportExportManager::ConversionResult {
     {
     }
 
-    ConversionResult(const QFuture<KisImportExportFilter::ConversionStatus> &futureStatus)
+    ConversionResult(const QFuture<KisImportExportErrorCode> &futureStatus)
         : m_isAsync(true),
           m_futureStatus(futureStatus)
     {
     }
 
-    ConversionResult(KisImportExportFilter::ConversionStatus status)
+    ConversionResult(KisImportExportErrorCode status)
         : m_isAsync(false),
           m_status(status)
     {
@@ -97,25 +98,25 @@ struct KisImportExportManager::ConversionResult {
         return m_isAsync;
     }
 
-    QFuture<KisImportExportFilter::ConversionStatus> futureStatus() const {
+    QFuture<KisImportExportErrorCode> futureStatus() const {
         // if the result is not async, then it means some failure happened,
         // just return a cancelled future
-        KIS_SAFE_ASSERT_RECOVER_NOOP(m_isAsync || m_status != KisImportExportFilter::OK);
+        KIS_SAFE_ASSERT_RECOVER_NOOP(m_isAsync || !m_status.isOk());
 
         return m_futureStatus;
     }
 
-    KisImportExportFilter::ConversionStatus status() const {
+    KisImportExportErrorCode status() const {
         return m_status;
     }
 
-    void setStatus(KisImportExportFilter::ConversionStatus value) {
+    void setStatus(KisImportExportErrorCode value) {
         m_status = value;
     }
 private:
     bool m_isAsync = false;
-    QFuture<KisImportExportFilter::ConversionStatus> m_futureStatus;
-    KisImportExportFilter::ConversionStatus m_status = KisImportExportFilter::UsageError;
+    QFuture<KisImportExportErrorCode> m_futureStatus;
+    KisImportExportErrorCode m_status = ImportExportCodes::InternalError;
 };
 
 
@@ -130,27 +131,28 @@ KisImportExportManager::~KisImportExportManager()
     delete d;
 }
 
-KisImportExportFilter::ConversionStatus KisImportExportManager::importDocument(const QString& location, const QString& mimeType)
+KisImportExportErrorCode KisImportExportManager::importDocument(const QString& location, const QString& mimeType)
 {
     ConversionResult result = convert(Import, location, location, mimeType, false, 0, false);
-    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!result.isAsync(), KisImportExportFilter::UsageError);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!result.isAsync(), ImportExportCodes::InternalError);
 
     return result.status();
 }
 
-KisImportExportFilter::ConversionStatus KisImportExportManager::exportDocument(const QString& location, const QString& realLocation, const QByteArray& mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
+KisImportExportErrorCode KisImportExportManager::exportDocument(const QString& location, const QString& realLocation, const QByteArray& mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
 {
     ConversionResult result = convert(Export, location, realLocation, mimeType, showWarnings, exportConfiguration, false);
-    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!result.isAsync(), KisImportExportFilter::UsageError);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!result.isAsync(), ImportExportCodes::InternalError);
 
     return result.status();
 }
 
-QFuture<KisImportExportFilter::ConversionStatus> KisImportExportManager::exportDocumentAsyc(const QString &location, const QString &realLocation, const QByteArray &mimeType, KisImportExportFilter::ConversionStatus &status, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
+QFuture<KisImportExportErrorCode> KisImportExportManager::exportDocumentAsyc(const QString &location, const QString &realLocation, const QByteArray &mimeType,
+                                                                            KisImportExportErrorCode &status, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
 {
     ConversionResult result = convert(Export, location, realLocation, mimeType, showWarnings, exportConfiguration, true);
     KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(result.isAsync() ||
-                                         result.status() != KisImportExportFilter::OK, QFuture<KisImportExportFilter::ConversionStatus>());
+                                         !result.status().isOk(), QFuture<KisImportExportErrorCode>());
 
     status = result.status();
     return result.futureStatus();
@@ -158,7 +160,7 @@ QFuture<KisImportExportFilter::ConversionStatus> KisImportExportManager::exportD
 
 // The static method to figure out to which parts of the
 // graph this mimetype has a connection to.
-QStringList KisImportExportManager::mimeFilter(Direction direction)
+QStringList KisImportExportManager::supportedMimeTypes(Direction direction)
 {
     // Find the right mimetype by the extension
     QSet<QString> mimeTypes;
@@ -277,7 +279,6 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
     // export configuration is supported for export only
     KIS_SAFE_ASSERT_RECOVER_NOOP(direction == Export || !bool(exportConfiguration));
 
-
     QString typeName = mimeType;
     if (typeName.isEmpty()) {
         typeName = KisMimeDatabase::mimeTypeForFile(location, direction == KisImportExportManager::Export ? false : true);
@@ -305,7 +306,7 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
     }
 
     if (!filter) {
-        return KisImportExportFilter::FilterCreationError;
+        return KisImportExportErrorCode(ImportExportCodes::FileFormatIncorrect);
     }
 
     filter->setFilename(location);
@@ -336,12 +337,20 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
 
     KIS_ASSERT_RECOVER_RETURN_VALUE(
                 direction == Import || direction == Export,
-                KisImportExportFilter::BadConversionGraph);
+                KisImportExportErrorCode(ImportExportCodes::InternalError)); // "bad conversion graph"
 
-
-
-    ConversionResult result = KisImportExportFilter::OK;
+    ConversionResult result = KisImportExportErrorCode(ImportExportCodes::OK);
     if (direction == Import) {
+
+        KisUsageLogger::log(QString("Importing %1 to %2. Location: %3. Real location: %4. Batchmode: %5")
+                            .arg(QString::fromLatin1(from))
+                            .arg(QString::fromLatin1(to))
+                            .arg(location)
+                            .arg(realLocation)
+                            .arg(batchMode()));
+
+
+
         // async importing is not yet supported!
         KIS_SAFE_ASSERT_RECOVER_NOOP(!isAsync);
 
@@ -366,15 +375,27 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
                                                        from, to,
                                                        batchMode(), showWarnings,
                                                        &alsoAsKra);
+
+
         if (!batchMode() && !askUser) {
-            return KisImportExportFilter::UserCancelled;
+            return KisImportExportErrorCode(ImportExportCodes::Cancelled);
         }
 
+        KisUsageLogger::log(QString("Converting from %1 to %2. Location: %3. Real location: %4. Batchmode: %5. Configuration: %6")
+                            .arg(QString::fromLatin1(from))
+                            .arg(QString::fromLatin1(to))
+                            .arg(location)
+                            .arg(realLocation)
+                            .arg(batchMode())
+                            .arg(exportConfiguration ? exportConfiguration->toXML() : "none"));
+
+
+
         if (isAsync) {
             result = QtConcurrent::run(std::bind(&KisImportExportManager::doExport, this, location, filter, exportConfiguration, alsoAsKra));
 
             // we should explicitly report that the exporting has been initiated
-            result.setStatus(KisImportExportFilter::OK);
+            result.setStatus(ImportExportCodes::OK);
 
         } else if (!batchMode()) {
             KisAsyncActionFeedback f(i18n("Saving document..."), 0);
@@ -383,17 +404,15 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
             result = doExport(location, filter, exportConfiguration, alsoAsKra);
         }
 
-        if (exportConfiguration) {
-            KisConfig().setExportConfiguration(typeName, exportConfiguration);
+        if (exportConfiguration && !batchMode() && showWarnings) {
+            KisConfig(false).setExportConfiguration(typeName, exportConfiguration);
         }
     }
     return result;
 }
 
-void KisImportExportManager::fillStaticExportConfigurationProperties(KisPropertiesConfigurationSP exportConfiguration)
+void KisImportExportManager::fillStaticExportConfigurationProperties(KisPropertiesConfigurationSP exportConfiguration, KisImageSP image)
 {
-    // Fill with some meta information about the image
-    KisImageSP image = m_document->image();
     KisPaintDeviceSP dev = image->projection();
     const KoColorSpace* cs = dev->colorSpace();
     const bool isThereAlpha =
@@ -409,8 +428,13 @@ void KisImportExportManager::fillStaticExportConfigurationProperties(KisProperti
     exportConfiguration->setProperty(KisImportExportFilter::sRGBTag, sRGB);
 }
 
-bool
-KisImportExportManager::askUserAboutExportConfiguration(
+
+void KisImportExportManager::fillStaticExportConfigurationProperties(KisPropertiesConfigurationSP exportConfiguration)
+{
+    return fillStaticExportConfigurationProperties(exportConfiguration, m_document->image());
+}
+
+bool KisImportExportManager::askUserAboutExportConfiguration(
         QSharedPointer<KisImportExportFilter> filter,
         KisPropertiesConfigurationSP exportConfiguration,
         const QByteArray &from,
@@ -420,6 +444,9 @@ KisImportExportManager::askUserAboutExportConfiguration(
         bool *alsoAsKra)
 {
 
+    // prevents the animation renderer from running this code
+
+
     const QString mimeUserDescription = KisMimeDatabase::descriptionForMimeType(to);
 
     QStringList warnings;
@@ -433,12 +460,19 @@ KisImportExportManager::askUserAboutExportConfiguration(
         errors = checker.errors();
     }
 
-    KisConfigWidget *wdg = filter->createConfigurationWidget(0, from, to);
+    KisConfigWidget *wdg = 0;
+
+    if (QThread::currentThread() == qApp->thread()) {
+        wdg = filter->createConfigurationWidget(0, from, to);
+    }
 
     // Extra checks that cannot be done by the checker, because the checker only has access to the image.
     if (!m_document->assistants().isEmpty() && to != m_document->nativeFormatMimeType()) {
         warnings.append(i18nc("image conversion warning", "The image contains <b>assistants</b>. The assistants will not be saved."));
     }
+    if (m_document->referenceImagesLayer() && m_document->referenceImagesLayer()->shapeCount() > 0 && to != m_document->nativeFormatMimeType()) {
+        warnings.append(i18nc("image conversion warning", "The image contains <b>reference images</b>. The reference images will not be saved."));
+    }
     if (m_document->guidesConfig().hasGuides() && to != m_document->nativeFormatMimeType()) {
         warnings.append(i18nc("image conversion warning", "The image contains <b>guides</b>. The guides will not be saved."));
     }
@@ -520,7 +554,7 @@ KisImportExportManager::askUserAboutExportConfiguration(
         QCheckBox *chkAlsoAsKra = 0;
         if (showWarnings && !warnings.isEmpty()) {
             chkAlsoAsKra = new QCheckBox(i18n("Also save your image as a Krita file."));
-            chkAlsoAsKra->setChecked(KisConfig().readEntry<bool>("AlsoSaveAsKra", false));
+            chkAlsoAsKra->setChecked(KisConfig(true).readEntry<bool>("AlsoSaveAsKra", false));
             layout->addWidget(chkAlsoAsKra);
         }
 
@@ -535,7 +569,7 @@ KisImportExportManager::askUserAboutExportConfiguration(
 
         *alsoAsKra = false;
         if (chkAlsoAsKra) {
-            KisConfig().writeEntry<bool>("AlsoSaveAsKra", chkAlsoAsKra->isChecked());
+            KisConfig(false).writeEntry<bool>("AlsoSaveAsKra", chkAlsoAsKra->isChecked());
             *alsoAsKra = chkAlsoAsKra->isChecked();
         }
 
@@ -547,19 +581,18 @@ KisImportExportManager::askUserAboutExportConfiguration(
     return true;
 }
 
-KisImportExportFilter::ConversionStatus KisImportExportManager::doImport(const QString &location, QSharedPointer<KisImportExportFilter> filter)
+KisImportExportErrorCode KisImportExportManager::doImport(const QString &location, QSharedPointer<KisImportExportFilter> filter)
 {
     QFile file(location);
     if (!file.exists()) {
-        return KisImportExportFilter::FileNotFound;
+        return ImportExportCodes::FileNotExist;
     }
 
     if (filter->supportsIO() && !file.open(QFile::ReadOnly)) {
-        return KisImportExportFilter::FileNotFound;
+        return KisImportExportErrorCode(KisImportExportErrorCannotRead(file.error()));
     }
 
-    KisImportExportFilter::ConversionStatus status =
-            filter->convert(m_document, &file, KisPropertiesConfigurationSP());
+    KisImportExportErrorCode status = filter->convert(m_document, &file, KisPropertiesConfigurationSP());
 
     if (file.isOpen()) {
         file.close();
@@ -568,12 +601,12 @@ KisImportExportFilter::ConversionStatus KisImportExportManager::doImport(const Q
     return status;
 }
 
-KisImportExportFilter::ConversionStatus KisImportExportManager::doExport(const QString &location, QSharedPointer<KisImportExportFilter> filter, KisPropertiesConfigurationSP exportConfiguration, bool alsoAsKra)
+KisImportExportErrorCode KisImportExportManager::doExport(const QString &location, QSharedPointer<KisImportExportFilter> filter, KisPropertiesConfigurationSP exportConfiguration, bool alsoAsKra)
 {
-    KisImportExportFilter::ConversionStatus status =
+    KisImportExportErrorCode status =
             doExportImpl(location, filter, exportConfiguration);
 
-    if (alsoAsKra && status == KisImportExportFilter::OK) {
+    if (alsoAsKra && status.isOk()) {
         QString kraLocation = location + ".kra";
         QByteArray mime = m_document->nativeFormatMimeType();
         QSharedPointer<KisImportExportFilter> filter(
@@ -589,35 +622,74 @@ KisImportExportFilter::ConversionStatus KisImportExportManager::doExport(const Q
 
             status = doExportImpl(kraLocation, filter, kraExportConfiguration);
         } else {
-            status = KisImportExportFilter::FilterCreationError;
+            status = ImportExportCodes::FileFormatIncorrect;
         }
     }
 
     return status;
 }
 
-KisImportExportFilter::ConversionStatus KisImportExportManager::doExportImpl(const QString &location, QSharedPointer<KisImportExportFilter> filter, KisPropertiesConfigurationSP exportConfiguration)
+// Temporary workaround until QTBUG-57299 is fixed.
+#ifndef Q_OS_WIN
+#define USE_QSAVEFILE
+#endif
+
+KisImportExportErrorCode KisImportExportManager::doExportImpl(const QString &location, QSharedPointer<KisImportExportFilter> filter, KisPropertiesConfigurationSP exportConfiguration)
 {
+#ifdef USE_QSAVEFILE
     QSaveFile file(location);
     file.setDirectWriteFallback(true);
-
     if (filter->supportsIO() && !file.open(QFile::WriteOnly)) {
+#else
+    QFileInfo fi(location);
+    QTemporaryFile file(fi.absolutePath() + ".XXXXXX.kra");
+    if (filter->supportsIO() && !file.open()) {
+#endif
+        KisImportExportErrorCannotWrite result(file.error());
+#ifdef USE_QSAVEFILE
         file.cancelWriting();
-        return KisImportExportFilter::CreationError;
+#endif
+        return result;
     }
 
-    KisImportExportFilter::ConversionStatus status =
-            filter->convert(m_document, &file, exportConfiguration);
+    KisImportExportErrorCode status = filter->convert(m_document, &file, exportConfiguration);
 
     if (filter->supportsIO()) {
-        if (status != KisImportExportFilter::OK) {
+        if (!status.isOk()) {
+#ifdef USE_QSAVEFILE
             file.cancelWriting();
+#endif
         } else {
+#ifdef USE_QSAVEFILE
             if (!file.commit()) {
-                status = KisImportExportFilter::CreationError;
+                qWarning() << "Could not commit QSaveFile";
+                status = KisImportExportErrorCannotWrite(file.error());
             }
+#else
+            file.flush();
+            file.close();
+            QFile target(location);
+            if (target.exists()) {
+                // There should already be a .kra~ backup
+                target.remove();
+            }
+            if (!file.copy(location)) {
+                file.setAutoRemove(false);
+                return KisImportExportErrorCannotWrite(file.error());
+            }
+#endif
         }
     }
+
+    // Do some minimal verification
+    QString verificationResult = filter->verify(location);
+    qDebug() << verificationResult;
+    if (!verificationResult.isEmpty()) {
+        status = KisImportExportErrorCode(ImportExportCodes::ErrorWhileWriting);
+        m_document->setErrorMessage(verificationResult);
+    }
+
+
     return status;
 
 }
