@@ -24,8 +24,8 @@
 #include <QTimer>
 #include <kundo2command.h>
 #include <QMimeData>
-
-#include <QTemporaryFile>
+#include <QApplication>
+#include <QThread>
 
 #include <KoShapeStroke.h>
 #include <KoPathShape.h>
@@ -82,6 +82,8 @@ KisShapeSelection::KisShapeSelection(KoShapeControllerBase *shapeControllerBase,
     m_model->moveToThread(image->thread());
     m_canvas->setObjectName("KisShapeSelectionCanvas");
     m_canvas->moveToThread(image->thread());
+
+    connect(this, SIGNAL(sigMoveShapes(QPointF)), SLOT(slotMoveShapes(QPointF)));
 }
 
 KisShapeSelection::~KisShapeSelection()
@@ -109,6 +111,12 @@ KisShapeSelection::KisShapeSelection(const KisShapeSelection& rhs, KisSelection*
 
 KisSelectionComponent* KisShapeSelection::clone(KisSelection* selection)
 {
+    /**
+     * TODO: make cloning of vector selections safe! Right now it crashes
+     * on Windows because of manipulations with timers from non-gui thread.
+     */
+    KIS_SAFE_ASSERT_RECOVER_NOOP(QThread::currentThread() == qApp->thread());
+
     return new KisShapeSelection(*this, selection);
 }
 
@@ -296,14 +304,24 @@ void KisShapeSelection::renderToProjection(KisPaintDeviceSP projection, const QR
     renderSelection(projection, r);
 }
 
-void KisShapeSelection::renderSelection(KisPaintDeviceSP projection, const QRect& r)
+void KisShapeSelection::renderSelection(KisPaintDeviceSP projection, const QRect& requestedRect)
 {
-    Q_ASSERT(projection);
-    Q_ASSERT(m_image);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(projection);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(m_image);
 
     const qint32 MASK_IMAGE_WIDTH = 256;
     const qint32 MASK_IMAGE_HEIGHT = 256;
 
+    const QPainterPath selectionOutline = outlineCache();
+
+    if (*projection->defaultPixel().data() == OPACITY_TRANSPARENT_U8) {
+        projection->clear(requestedRect);
+    } else {
+        KoColor transparentColor(Qt::transparent, projection->colorSpace());
+        projection->fill(requestedRect, transparentColor);
+    }
+    const QRect r = requestedRect & selectionOutline.boundingRect().toAlignedRect();
+
     QImage polygonMaskImage(MASK_IMAGE_WIDTH, MASK_IMAGE_HEIGHT, QImage::Format_ARGB32);
     QPainter maskPainter(&polygonMaskImage);
     maskPainter.setRenderHint(QPainter::Antialiasing, true);
@@ -314,7 +332,7 @@ void KisShapeSelection::renderSelection(KisPaintDeviceSP projection, const QRect
 
             maskPainter.fillRect(polygonMaskImage.rect(), Qt::black);
             maskPainter.translate(-x, -y);
-            maskPainter.fillPath(outlineCache(), Qt::white);
+            maskPainter.fillPath(selectionOutline, Qt::white);
             maskPainter.translate(x, y);
 
             qint32 rectWidth = qMin(r.x() + r.width() - x, MASK_IMAGE_WIDTH);
@@ -341,20 +359,22 @@ KisShapeSelectionFactory::KisShapeSelectionFactory()
 
 void KisShapeSelection::moveX(qint32 x)
 {
-    Q_FOREACH (KoShape* shape, shapeManager()->shapes()) {
-        if (shape != this) {
-            QPointF pos = shape->position();
-            shape->setPosition(QPointF(pos.x() + x/m_image->xRes(), pos.y()));
-        }
-    }
+    const QPointF diff(x / m_image->xRes(), 0);
+    emit sigMoveShapes(diff);
 }
 
 void KisShapeSelection::moveY(qint32 y)
+{
+    const QPointF diff(0, y / m_image->yRes());
+    emit sigMoveShapes(diff);
+}
+
+void KisShapeSelection::slotMoveShapes(const QPointF &diff)
 {
     Q_FOREACH (KoShape* shape, shapeManager()->shapes()) {
         if (shape != this) {
             QPointF pos = shape->position();
-            shape->setPosition(QPointF(pos.x(), pos.y() + y/m_image->yRes()));
+            shape->setPosition(pos + diff);
         }
     }
 }
