@@ -22,6 +22,7 @@
 #include <QMessageBox>
 #include <QSignalMapper>
 #include <QApplication>
+#include <QMessageBox>
 
 #include <kactioncollection.h>
 
@@ -72,6 +73,7 @@
 #include "kis_node_selection_adapter.h"
 #include "kis_node_insertion_adapter.h"
 #include "kis_node_juggler_compressed.h"
+#include "KisNodeDisplayModeAdapter.h"
 #include "kis_clipboard.h"
 #include "kis_node_dummies_graph.h"
 #include "kis_mimedata.h"
@@ -83,6 +85,8 @@
 #include "KisView.h"
 
 #include <kis_signals_blocker.h>
+#include <libs/image/kis_layer_properties_icons.h>
+#include <libs/image/commands/kis_node_property_list_command.h>
 
 struct KisNodeManager::Private {
 
@@ -95,6 +99,8 @@ struct KisNodeManager::Private {
         , commandsAdapter(v)
         , nodeSelectionAdapter(new KisNodeSelectionAdapter(q))
         , nodeInsertionAdapter(new KisNodeInsertionAdapter(q))
+        , nodeDisplayModeAdapter(new KisNodeDisplayModeAdapter())
+        , lastRequestedIsolatedModeStatus(false)
     {
     }
 
@@ -106,6 +112,7 @@ struct KisNodeManager::Private {
     KisNodeCommandsAdapter commandsAdapter;
     QScopedPointer<KisNodeSelectionAdapter> nodeSelectionAdapter;
     QScopedPointer<KisNodeInsertionAdapter> nodeInsertionAdapter;
+    QScopedPointer<KisNodeDisplayModeAdapter> nodeDisplayModeAdapter;
 
     KisAction *showInTimeline;
 
@@ -119,6 +126,8 @@ struct KisNodeManager::Private {
     QSignalMapper nodeCreationSignalMapper;
     QSignalMapper nodeConversionSignalMapper;
 
+    bool lastRequestedIsolatedModeStatus;
+
     void saveDeviceAsImage(KisPaintDeviceSP device,
                            const QString &defaultName,
                            const QRect &bounds,
@@ -181,7 +190,6 @@ bool KisNodeManager::Private::activateNodeImpl(KisNodeSP node)
             // XXX_NODE: for now, masks cannot be nested.
             layerManager.activateLayer(static_cast<KisLayer*>(node->parent().data()));
         }
-
     }
     return true;
 }
@@ -216,8 +224,8 @@ void KisNodeManager::setView(QPointer<KisView>imageView)
         KisShapeController *shapeController = dynamic_cast<KisShapeController*>(m_d->imageView->document()->shapeController());
         Q_ASSERT(shapeController);
         connect(shapeController, SIGNAL(sigActivateNode(KisNodeSP)), SLOT(slotNonUiActivatedNode(KisNodeSP)));
-        connect(m_d->imageView->image(), SIGNAL(sigIsolatedModeChanged()),this, SLOT(slotUpdateIsolateModeAction()));
-        connect(m_d->imageView->image(), SIGNAL(sigRequestNodeReselection(KisNodeSP, const KisNodeList&)),this, SLOT(slotImageRequestNodeReselection(KisNodeSP, const KisNodeList&)));
+        connect(m_d->imageView->image(), SIGNAL(sigIsolatedModeChanged()),this, SLOT(slotUpdateIsolateModeActionImageStatusChange()));
+        connect(m_d->imageView->image(), SIGNAL(sigRequestNodeReselection(KisNodeSP,KisNodeList)),this, SLOT(slotImageRequestNodeReselection(KisNodeSP,KisNodeList)));
         m_d->imageView->resourceProvider()->slotNodeActivated(m_d->imageView->currentNode());
     }
 
@@ -249,12 +257,20 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     m_d->layerManager.setup(actionManager);
     m_d->maskManager.setup(actionCollection, actionManager);
 
-    KisAction * action  = actionManager->createAction("mirrorNodeX");
+    KisAction * action = 0;
+
+    action = actionManager->createAction("mirrorNodeX");
     connect(action, SIGNAL(triggered()), this, SLOT(mirrorNodeX()));
 
     action  = actionManager->createAction("mirrorNodeY");
     connect(action, SIGNAL(triggered()), this, SLOT(mirrorNodeY()));
 
+    action = actionManager->createAction("mirrorAllNodesX");
+    connect(action, SIGNAL(triggered()), this, SLOT(mirrorAllNodesX()));
+
+    action  = actionManager->createAction("mirrorAllNodesY");
+    connect(action, SIGNAL(triggered()), this, SLOT(mirrorAllNodesY()));
+
     action = actionManager->createAction("activateNextLayer");
     connect(action, SIGNAL(triggered()), this, SLOT(activateNextNode()));
 
@@ -339,8 +355,8 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
 
     NEW_LAYER_ACTION("add_new_selection_mask", "KisSelectionMask");
 
-    connect(&m_d->nodeCreationSignalMapper, SIGNAL(mapped(const QString &)),
-            this, SLOT(createNode(const QString &)));
+    connect(&m_d->nodeCreationSignalMapper, SIGNAL(mapped(QString)),
+            this, SLOT(createNode(QString)));
 
     CONVERT_NODE_ACTION("convert_to_paint_layer", "KisPaintLayer");
 
@@ -352,10 +368,10 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
 
     CONVERT_NODE_ACTION("convert_to_animated", "animated");
 
-    CONVERT_NODE_ACTION_2("convert_layer_to_file_layer", "KisFileLayer", QStringList()<< "KisFileLayer" << "KisCloneLayer");
+    CONVERT_NODE_ACTION_2("convert_to_file_layer", "KisFileLayer", QStringList() << "KisGroupLayer" << "KisFileLayer" << "KisCloneLayer");
 
-    connect(&m_d->nodeConversionSignalMapper, SIGNAL(mapped(const QString &)),
-            this, SLOT(convertNode(const QString &)));
+    connect(&m_d->nodeConversionSignalMapper, SIGNAL(mapped(QString)),
+            this, SLOT(convertNode(QString)));
 
     action = actionManager->createAction("isolate_layer");
     connect(action, SIGNAL(triggered(bool)), this, SLOT(toggleIsolateMode(bool)));
@@ -485,6 +501,20 @@ void KisNodeManager::toggleIsolateMode(bool checked)
     } else {
         image->stopIsolatedMode();
     }
+
+    m_d->lastRequestedIsolatedModeStatus = checked;
+}
+
+void KisNodeManager::slotUpdateIsolateModeActionImageStatusChange()
+{
+    slotUpdateIsolateModeAction();
+
+    KisNodeSP isolatedRootNode = m_d->view->image()->isolatedModeRoot();
+    if (this->activeNode() &&
+        bool(isolatedRootNode) != m_d->lastRequestedIsolatedModeStatus) {
+
+        slotTryRestartIsolatedMode();
+    }
 }
 
 void KisNodeManager::slotUpdateIsolateModeAction()
@@ -500,8 +530,14 @@ void KisNodeManager::slotUpdateIsolateModeAction()
 
 void KisNodeManager::slotTryRestartIsolatedMode()
 {
+    /**
+     * It might be that we have multiple Krita windows open. In such a case
+     * only the currently active one should restart isolated mode
+     */
+    if (!m_d->view->mainWindow()->isActiveWindow()) return;
+
     KisNodeSP isolatedRootNode = m_d->view->image()->isolatedModeRoot();
-    if (!isolatedRootNode) return;
+    if (!isolatedRootNode && !m_d->lastRequestedIsolatedModeStatus) return;
 
     this->toggleIsolateMode(true);
 }
@@ -564,16 +600,16 @@ void KisNodeManager::slotShowHideTimeline(bool value)
 
 KisLayerSP KisNodeManager::createPaintLayer()
 {
-    KisNodeSP activeNode = this->activeNode();
-    if (!activeNode) {
-        activeNode = m_d->view->image()->root();
-    }
-
-    return m_d->layerManager.addPaintLayer(activeNode);
+    KisNodeSP node = createNode("KisPaintLayer");
+    return dynamic_cast<KisLayer*>(node.data());
 }
 
 void KisNodeManager::convertNode(const QString &nodeType)
 {
+    if (!m_d->view->blockUntilOperationsFinished(m_d->view->image())) {
+        return;
+    }
+
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) return;
 
@@ -614,6 +650,14 @@ void KisNodeManager::convertNode(const QString &nodeType)
 
 void KisNodeManager::slotSomethingActivatedNodeImpl(KisNodeSP node)
 {
+    KisDummiesFacadeBase *dummiesFacade = dynamic_cast<KisDummiesFacadeBase*>(m_d->imageView->document()->shapeController());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(dummiesFacade);
+
+    const bool nodeVisible = !isNodeHidden(node, !m_d->nodeDisplayModeAdapter->showGlobalSelectionMask());
+    if (!nodeVisible) {
+        return;
+    }
+
     KIS_ASSERT_RECOVER_RETURN(node != activeNode());
     if (m_d->activateNodeImpl(node)) {
         emit sigUiNeedChangeActiveNode(node);
@@ -672,7 +716,12 @@ void KisNodeManager::slotUiActivatedNode(KisNodeSP node)
                 << "KritaFill/KisToolFill"
                 << "KritaFill/KisToolGradient";
 
-        if (node->inherits("KisShapeLayer")) {
+
+        KisSelectionMask *selectionMask = dynamic_cast<KisSelectionMask*>(node.data());
+        const bool nodeHasVectorAbilities = node->inherits("KisShapeLayer") ||
+            (selectionMask && selectionMask->selection()->hasShapeSelection());
+
+        if (nodeHasVectorAbilities) {
             if (pixelTools.contains(KoToolManager::instance()->activeToolId())) {
                 KoToolManager::instance()->switchToolRequested("InteractionTool");
             }
@@ -723,6 +772,11 @@ void KisNodeManager::nodeProperties(KisNodeSP node)
     }
 }
 
+void KisNodeManager::changeCloneSource()
+{
+    m_d->layerManager.changeCloneSource();
+}
+
 qint32 KisNodeManager::convertOpacityToInt(qreal opacity)
 {
     /**
@@ -733,18 +787,21 @@ qint32 KisNodeManager::convertOpacityToInt(qreal opacity)
     return qMin(255, int(opacity * 2.55 + 0.5));
 }
 
-void KisNodeManager::setNodeOpacity(KisNodeSP node, qint32 opacity,
-                                    bool finalChange)
+void KisNodeManager::setNodeName(KisNodeSP node, const QString &name)
+{
+    if (!node) return;
+    if (node->name() == name) return;
+
+    m_d->commandsAdapter.setNodeName(node, name);
+
+}
+
+void KisNodeManager::setNodeOpacity(KisNodeSP node, qint32 opacity)
 {
     if (!node) return;
     if (node->opacity() == opacity) return;
 
-    if (!finalChange) {
-        node->setOpacity(opacity);
-        node->setDirty();
-    } else {
-        m_d->commandsAdapter.setOpacity(node, opacity);
-    }
+    m_d->commandsAdapter.setOpacity(node, opacity);
 }
 
 void KisNodeManager::setNodeCompositeOp(KisNodeSP node,
@@ -787,9 +844,14 @@ KisNodeInsertionAdapter* KisNodeManager::nodeInsertionAdapter() const
     return m_d->nodeInsertionAdapter.data();
 }
 
+KisNodeDisplayModeAdapter *KisNodeManager::nodeDisplayModeAdapter() const
+{
+    return m_d->nodeDisplayModeAdapter.data();
+}
+
 bool KisNodeManager::isNodeHidden(KisNodeSP node, bool isGlobalSelectionHidden)
 {
-    if (dynamic_cast<KisReferenceImagesLayer *>(node.data())) {
+    if (node && node->isFakeNode()) {
         return true;
     }
 
@@ -801,11 +863,31 @@ bool KisNodeManager::isNodeHidden(KisNodeSP node, bool isGlobalSelectionHidden)
     return false;
 }
 
-void KisNodeManager::nodeOpacityChanged(qreal opacity, bool finalChange)
+bool KisNodeManager::trySetNodeProperties(KisNodeSP node, KisImageSP image, KisBaseNode::PropertyList properties) const
+{
+    const KisPaintLayer *paintLayer = dynamic_cast<KisPaintLayer*>(node.data());
+    if (paintLayer) {
+        const auto onionSkinOn = KisLayerPropertiesIcons::getProperty(KisLayerPropertiesIcons::onionSkins, true);
+
+        if (properties.contains(onionSkinOn)) {
+            const KisPaintDeviceSP &paintDevice = paintLayer->paintDevice();
+            if (paintDevice && paintDevice->defaultPixel().opacityU8() == 255) {
+                m_d->view->showFloatingMessage(i18n("Onion skins require a layer with transparent background."), QIcon());
+                return false;
+            }
+        }
+    }
+
+    KisNodePropertyListCommand::setNodePropertiesNoUndo(node, image, properties);
+
+    return true;
+}
+
+void KisNodeManager::nodeOpacityChanged(qreal opacity)
 {
     KisNodeSP node = activeNode();
 
-    setNodeOpacity(node, convertOpacityToInt(opacity), finalChange);
+    setNodeOpacity(node, convertOpacityToInt(opacity));
 }
 
 void KisNodeManager::nodeCompositeOpChanged(const KoCompositeOp* op)
@@ -885,7 +967,7 @@ void KisNodeManager::mirrorNodeX()
     } else if (node->inherits("KisMask")) {
         commandName = kundo2_i18n("Mirror Mask X");
     }
-    mirrorNode(node, commandName, Qt::Horizontal);
+    mirrorNode(node, commandName, Qt::Horizontal, m_d->view->selection());
 }
 
 void KisNodeManager::mirrorNodeY()
@@ -898,7 +980,21 @@ void KisNodeManager::mirrorNodeY()
     } else if (node->inherits("KisMask")) {
         commandName = kundo2_i18n("Mirror Mask Y");
     }
-    mirrorNode(node, commandName, Qt::Vertical);
+    mirrorNode(node, commandName, Qt::Vertical, m_d->view->selection());
+}
+
+void KisNodeManager::mirrorAllNodesX()
+{
+    KisNodeSP node = m_d->view->image()->root();
+    mirrorNode(node, kundo2_i18n("Mirror All Layers X"),
+               Qt::Horizontal, m_d->view->selection());
+}
+
+void KisNodeManager::mirrorAllNodesY()
+{
+    KisNodeSP node = m_d->view->image()->root();
+    mirrorNode(node, kundo2_i18n("Mirror All Layers Y"),
+               Qt::Vertical, m_d->view->selection());
 }
 
 void KisNodeManager::activateNextNode()
@@ -960,56 +1056,10 @@ void KisNodeManager::switchToPreviouslyActiveNode()
     }
 }
 
-void KisNodeManager::rotate(double radians)
-{
-    if(!m_d->view->image()) return;
-
-    KisNodeSP node = activeNode();
-    if (!node) return;
-
-    if (!m_d->view->blockUntilOperationsFinished(m_d->view->image())) return;
-
-    m_d->view->image()->rotateNode(node, radians);
-}
-
-void KisNodeManager::rotate180()
-{
-    rotate(M_PI);
-}
-
-void KisNodeManager::rotateLeft90()
-{
-   rotate(-M_PI / 2);
-}
-
-void KisNodeManager::rotateRight90()
-{
-    rotate(M_PI / 2);
-}
-
-void KisNodeManager::shear(double angleX, double angleY)
-{
-    if (!m_d->view->image()) return;
-
-    KisNodeSP node = activeNode();
-    if (!node) return;
-
-    if(!m_d->view->blockUntilOperationsFinished(m_d->view->image())) return;
-
-    m_d->view->image()->shearNode(node, angleX, angleY);
-}
-
-void KisNodeManager::scale(double sx, double sy, KisFilterStrategy *filterStrategy)
-{
-    KisNodeSP node = activeNode();
-    KIS_ASSERT_RECOVER_RETURN(node);
-
-    m_d->view->image()->scaleNode(node, sx, sy, filterStrategy);
-
-    nodesUpdated();
-}
-
-void KisNodeManager::mirrorNode(KisNodeSP node, const KUndo2MagicString& actionName, Qt::Orientation orientation)
+void KisNodeManager::mirrorNode(KisNodeSP node,
+                                const KUndo2MagicString& actionName,
+                                Qt::Orientation orientation,
+                                KisSelectionSP selection)
 {
     KisImageSignalVector emitSignals;
     emitSignals << ModifiedSignal;
@@ -1018,10 +1068,20 @@ void KisNodeManager::mirrorNode(KisNodeSP node, const KUndo2MagicString& actionN
                                        KisProcessingApplicator::RECURSIVE,
                                        emitSignals, actionName);
 
-    KisProcessingVisitorSP visitor =
-        new KisMirrorProcessingVisitor(m_d->view->image()->bounds(), orientation);
+    KisProcessingVisitorSP visitor;
+
+    if (selection) {
+        visitor = new KisMirrorProcessingVisitor(selection, orientation);
+    } else {
+        visitor = new KisMirrorProcessingVisitor(m_d->view->image()->bounds(), orientation);
+    }
+
+    if (!selection) {
+        applicator.applyVisitorAllFrames(visitor, KisStrokeJobData::CONCURRENT);
+    } else {
+        applicator.applyVisitor(visitor, KisStrokeJobData::CONCURRENT);
+    }
 
-    applicator.applyVisitor(visitor, KisStrokeJobData::CONCURRENT);
     applicator.end();
 
     nodesUpdated();
@@ -1063,7 +1123,13 @@ void KisNodeManager::Private::saveDeviceAsImage(KisPaintDeviceSP device,
 
     dst->initialRefreshGraph();
 
-    doc->exportDocumentSync(url, mimefilter.toLatin1());
+    if (!doc->exportDocumentSync(url, mimefilter.toLatin1())) {
+        QMessageBox::warning(0,
+                             i18nc("@title:window", "Krita"),
+                             i18n("Could not save the layer. %1", doc->errorMessage().toUtf8().data()),
+                             QMessageBox::Ok);
+
+    }
 }
 
 void KisNodeManager::saveNodeAsImage()
@@ -1078,12 +1144,8 @@ void KisNodeManager::saveNodeAsImage()
     KisImageWSP image = m_d->view->image();
     QRect saveRect = image->bounds() | node->exactBounds();
 
-    KisPaintDeviceSP device = node->paintDevice();
-    if (!device) {
-        device = node->projection();
-    }
-
-    m_d->saveDeviceAsImage(device, node->name(),
+    m_d->saveDeviceAsImage(node->projection(),
+                           node->name(),
                            saveRect,
                            image->xRes(), image->yRes(),
                            node->opacity());
@@ -1373,6 +1435,8 @@ void KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
     nodes2 = KisLayerUtils::sortMergableNodes(image->root(), selectedNodes());
     KisLayerUtils::filterMergableNodes(nodes2);
 
+    if (nodes2.size() == 0) return;
+
     if (KisLayerUtils::checkIsChildOf(active, nodes2)) {
         active = nodes2.first();
     }
