@@ -24,7 +24,7 @@
 #include <KoColor.h>
 #include <KoToolBase.h>
 #include <KoID.h>
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
 #include <kritaui_export.h>
 #include <kis_types.h>
 
@@ -161,6 +161,14 @@ public:
         NONE = 10000
     };
 
+    enum NodePaintAbility {
+        VECTOR,
+        CLONE,
+        PAINT,
+        UNPAINTABLE
+    };
+    Q_ENUMS(NodePaintAbility)
+
     static AlternateAction actionToAlternateAction(ToolAction action);
 
     virtual void activateAlternateAction(AlternateAction action);
@@ -180,6 +188,8 @@ public:
 
     bool isActive() const;
 
+    KisTool::NodePaintAbility nodePaintAbility();
+
 public Q_SLOTS:
     void activate(ToolActivation activation, const QSet<KoShape*> &shapes) override;
     void deactivate() override;
@@ -191,7 +201,7 @@ public Q_SLOTS:
     virtual void updateSettingsViews();
 
 Q_SIGNALS:
-    void isActiveChanged();
+    void isActiveChanged(bool isActivated);
 
 protected:
     // conversion methods are also needed by the paint information builder
@@ -216,6 +226,7 @@ protected:
     QPoint convertToImagePixelCoordFloored(KoPointerEvent *e);
 
     QRectF convertToPt(const QRectF &rect);
+    qreal convertToPt(qreal value);
 
     QPointF viewToPixel(const QPointF &viewCoord) const;
     /// Convert an integer pixel coordinate into a view coordinate.
@@ -287,14 +298,15 @@ protected:
     void blockUntilOperationsFinishedForced();
 
 protected:
-    enum ToolMode {
+    enum ToolMode: int {
         HOVER_MODE,
         PAINT_MODE,
         SECONDARY_PAINT_MODE,
         MIRROR_AXIS_SETUP_MODE,
         GESTURE_MODE,
         PAN_MODE,
-        OTHER // not used now
+        OTHER, // tool-specific modes, like multibrush's symmetry axis setup
+        OTHER_1
     };
 
     virtual void setMode(ToolMode mode);
@@ -307,10 +319,6 @@ protected Q_SLOTS:
      */
     virtual void resetCursorStyle();
 
-private Q_SLOTS:
-    void slotToggleFgBg();
-    void slotResetFgBg();
-
 private:
     struct Private;
     Private* const d;
