@@ -442,13 +442,13 @@ bool EllipseShape::saveSvg(SvgSavingContext &context)
         SvgUtil::writeTransformAttributeLazy("transform", transformation(), context.shapeWriter());
 
         if (isCircle) {
-            context.shapeWriter().addAttributePt("r", 0.5 * size.width());
+            context.shapeWriter().addAttribute("r", 0.5 * size.width());
         } else {
-            context.shapeWriter().addAttributePt("rx", 0.5 * size.width());
-            context.shapeWriter().addAttributePt("ry", 0.5 * size.height());
+            context.shapeWriter().addAttribute("rx", 0.5 * size.width());
+            context.shapeWriter().addAttribute("ry", 0.5 * size.height());
         }
-        context.shapeWriter().addAttributePt("cx", 0.5 * size.width());
-        context.shapeWriter().addAttributePt("cy", 0.5 * size.height());
+        context.shapeWriter().addAttribute("cx", 0.5 * size.width());
+        context.shapeWriter().addAttribute("cy", 0.5 * size.height());
 
         SvgStyleWriter::saveSvgStyle(this, context);
 
@@ -460,11 +460,11 @@ bool EllipseShape::saveSvg(SvgSavingContext &context)
 
         context.shapeWriter().addAttribute("sodipodi:type", "arc");
 
-        context.shapeWriter().addAttributePt("sodipodi:rx", m_radii.x());
-        context.shapeWriter().addAttributePt("sodipodi:ry", m_radii.y());
+        context.shapeWriter().addAttribute("sodipodi:rx", m_radii.x());
+        context.shapeWriter().addAttribute("sodipodi:ry", m_radii.y());
 
-        context.shapeWriter().addAttributePt("sodipodi:cx", m_center.x());
-        context.shapeWriter().addAttributePt("sodipodi:cy", m_center.y());
+        context.shapeWriter().addAttribute("sodipodi:cx", m_center.x());
+        context.shapeWriter().addAttribute("sodipodi:cy", m_center.y());
 
         context.shapeWriter().addAttribute("sodipodi:start", 2 * M_PI - kisDegreesToRadians(endAngle()));
         context.shapeWriter().addAttribute("sodipodi:end", 2 * M_PI - kisDegreesToRadians(startAngle()));
