@@ -32,7 +32,6 @@ KisDynamicSensorDrawingAngle::KisDynamicSensorDrawingAngle()
       m_fanCornersEnabled(false),
       m_fanCornersStep(30),
       m_angleOffset(0),
-      m_dabIndex(0),
       m_lockedAngle(0),
       m_lockedAngleMode(false)
 {
@@ -40,27 +39,17 @@ KisDynamicSensorDrawingAngle::KisDynamicSensorDrawingAngle()
 
 void KisDynamicSensorDrawingAngle::reset()
 {
-    m_dabIndex = 0;
 }
 
 qreal KisDynamicSensorDrawingAngle::value(const KisPaintInformation& info)
 {
     /* so that we are in 0.0..1.0 */
-    qreal ret = 0.5 + info.drawingAngle() / (2.0 * M_PI) + m_angleOffset/360.0;
+    qreal ret = 0.5 + info.drawingAngle(m_lockedAngleMode) / (2.0 * M_PI) + m_angleOffset/360.0;
 
     // check if m_angleOffset pushed us out of bounds
     if (ret > 1.0)
         ret -= 1.0;
 
-    if (!info.isHoveringMode() && m_lockedAngleMode) {
-        if (!m_dabIndex) {
-            info.lockCurrentDrawingAngle(1.0);
-        } else {
-            info.lockCurrentDrawingAngle(0.5);
-        }
-        m_dabIndex++;
-    }
-
     return ret;
 }
 
@@ -166,7 +155,7 @@ void KisDynamicSensorDrawingAngle::setLockedAngleMode(int value)
 
 void KisDynamicSensorDrawingAngle::setAngleOffset(int angle)
 {
-    Q_ASSERT(angle >= 0 && angle < 360);//dont include 360
+    Q_ASSERT(angle >= 0 && angle < 360);//don't include 360
     m_angleOffset = angle;
 }
 
