@@ -24,7 +24,6 @@
 #include <QApplication>
 #include <QBuffer>
 #include <QComboBox>
-#include <QDebug>
 #include <QDialogButtonBox>
 #include <QDoubleSpinBox>
 #include <QFontComboBox>
@@ -64,6 +63,7 @@
 #include <KoColorPopupAction.h>
 #include <svg/SvgUtil.h>
 
+#include <KisScreenColorPicker.h>
 #include <kis_icon.h>
 #include <kis_config.h>
 #include <kis_file_name_requester.h>
@@ -92,7 +92,7 @@ SvgTextEditor::SvgTextEditor(QWidget *parent, Qt::WindowFlags flags)
     m_charSelectDialog->setButtons(KoDialog::Close);
 #endif
     connect(m_textEditorWidget.buttons, SIGNAL(accepted()), this, SLOT(save()));
-    connect(m_textEditorWidget.buttons, SIGNAL(rejected()), this, SLOT(close()));
+    connect(m_textEditorWidget.buttons, SIGNAL(rejected()), this, SLOT(slotCloseEditor()));
     connect(m_textEditorWidget.buttons, SIGNAL(clicked(QAbstractButton*)), this, SLOT(dialogButtonClicked(QAbstractButton*)));
 
     KConfigGroup cg(KSharedConfig::openConfig(), "SvgTextTool");
@@ -110,7 +110,7 @@ SvgTextEditor::SvgTextEditor(QWidget *parent, Qt::WindowFlags flags)
 
     setAcceptDrops(true);
     //setStandardToolBarMenuEnabled(true);
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     setUnifiedTitleAndToolBarOnMac(true);
 #endif
     setTabPosition(Qt::AllDockWidgetAreas, QTabWidget::North);
@@ -146,7 +146,6 @@ SvgTextEditor::SvgTextEditor(QWidget *parent, Qt::WindowFlags flags)
     m_textEditorWidget.richTextEdit->document()->setDefaultStyleSheet("p {margin:0px;}");
 
     applySettings();
-
 }
 
 SvgTextEditor::~SvgTextEditor()
@@ -171,8 +170,19 @@ void SvgTextEditor::setShape(KoSvgTextShape *shape)
             m_textEditorWidget.svgTextEdit->setPlainText(svg);
             m_textEditorWidget.svgStylesEdit->setPlainText(styles);
             m_textEditorWidget.svgTextEdit->document()->setModified(false);
-            if (converter.convertSvgToDocument(svg, doc)) {
+
+            if (shape->isRichTextPreferred() &&
+                converter.convertSvgToDocument(svg, doc)) {
+
                 m_textEditorWidget.richTextEdit->setDocument(doc);
+                KisSignalsBlocker b(m_textEditorWidget.textTab);
+                m_textEditorWidget.textTab->setCurrentIndex(Richtext);
+                doc->clearUndoRedoStacks();
+                switchTextEditorTab(false);
+            } else {
+                KisSignalsBlocker b(m_textEditorWidget.textTab);
+                m_textEditorWidget.textTab->setCurrentIndex(SvgSource);
+                switchTextEditorTab(false);
             }
         }
         else {
@@ -195,17 +205,17 @@ void SvgTextEditor::save()
                     qWarning()<<"new converter doesn't work!";
             }
             m_textEditorWidget.richTextEdit->document()->setModified(false);
-            emit textUpdated(m_shape, svg, styles);
+            emit textUpdated(m_shape, svg, styles, true);
         }
         else {
-            emit textUpdated(m_shape, m_textEditorWidget.svgTextEdit->document()->toPlainText(), m_textEditorWidget.svgStylesEdit->document()->toPlainText());
+            emit textUpdated(m_shape, m_textEditorWidget.svgTextEdit->document()->toPlainText(), m_textEditorWidget.svgStylesEdit->document()->toPlainText(), false);
             m_textEditorWidget.svgTextEdit->document()->setModified(false);
         }
     }
 
 }
 
-void SvgTextEditor::switchTextEditorTab()
+void SvgTextEditor::switchTextEditorTab(bool convertData)
 {
     KoSvgTextShape shape;
     KoSvgTextShapeMarkupConverter converter(&shape);
@@ -217,41 +227,30 @@ void SvgTextEditor::switchTextEditorTab()
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
         //first, make buttons checkable
         enableRichTextActions(true);
+        enableSvgTextActions(false);
 
         //then connect the cursor change to the checkformat();
         connect(m_textEditorWidget.richTextEdit, SIGNAL(cursorPositionChanged()), this, SLOT(checkFormat()));
 
 
-        if (m_shape) {
-
-            // Convert the svg text to html XXX: Fix resolution! Also, the rect should be the image rect, not the shape rect.
-            /**
-            if (!converter.convertFromSvg(m_textEditorWidget.svgTextEdit->document()->toPlainText(), m_textEditorWidget.svgStylesEdit->document()->toPlainText(),
-                                          m_shape->boundingRect(), 72.0)) {
-                qDebug() << "Eeek 3";
-            }
-            QString html;
-            if (!converter.convertToHtml(&html)) {
-                qDebug() << "Eeek 4";
-            }
-
-            m_textEditorWidget.richTextEdit->document()->setHtml(html);
-            */
+        if (m_shape && convertData) {
             QTextDocument *doc = m_textEditorWidget.richTextEdit->document();
             if (!converter.convertSvgToDocument(m_textEditorWidget.svgTextEdit->document()->toPlainText(), doc)) {
                 qWarning()<<"new converter svgToDoc doesn't work!";
             }
             m_textEditorWidget.richTextEdit->setDocument(doc);
+            doc->clearUndoRedoStacks();
         }
         m_currentEditor = m_textEditorWidget.richTextEdit;
     }
     else {
         //first, make buttons uncheckable
         enableRichTextActions(false);
+        enableSvgTextActions(true);
         disconnect(m_textEditorWidget.richTextEdit, SIGNAL(cursorPositionChanged()), this, SLOT(checkFormat()));
 
         // Convert the rich text to svg and styles strings
-        if (m_shape) {
+        if (m_shape && convertData) {
             QString svg;
             QString styles;
 
@@ -271,6 +270,11 @@ void SvgTextEditor::checkFormat()
     QTextCharFormat format = m_textEditorWidget.richTextEdit->textCursor().charFormat();
     QTextBlockFormat blockFormat = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
 
+    /**
+     * Make sure that when we remove the very last symbol, the last used font will not change
+     */
+    m_textEditorWidget.richTextEdit->document()->setDefaultFont(format.font());
+
     // checkboxes do not emit signals on manual switching, so we
     // can avoid blocking them
 
@@ -320,6 +324,12 @@ void SvgTextEditor::checkFormat()
             spnLineHeight->setValue(double(blockFormat.lineHeight()));
         }
     }
+
+    {
+        QDoubleSpinBox* spnLetterSpacing = qobject_cast<QDoubleSpinBox*>(qobject_cast<QWidgetAction*>(actionCollection()->action("svg_letter_spacing"))->defaultWidget());
+        KisSignalsBlocker b(spnLetterSpacing);
+        spnLetterSpacing->setValue(format.fontLetterSpacing());
+    }
 }
 
 void SvgTextEditor::undo()
@@ -456,12 +466,14 @@ void SvgTextEditor::setTextBold(QFont::Weight weight)
 {
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
         QTextCharFormat format;
+        QTextCursor oldCursor = setTextSelection();
         if (m_textEditorWidget.richTextEdit->textCursor().charFormat().fontWeight() > QFont::Normal && weight==QFont::Bold) {
             format.setFontWeight(QFont::Normal);
         } else {
             format.setFontWeight(weight);
         }
         m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
     } else {
         QTextCursor cursor = m_textEditorWidget.svgTextEdit->textCursor();
         if (cursor.hasSelection()) {
@@ -517,8 +529,10 @@ void SvgTextEditor::setTextItalic(QFont::Style style)
 
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
         QTextCharFormat format;
+        QTextCursor origCursor = setTextSelection();
         format.setFontItalic(!m_textEditorWidget.richTextEdit->textCursor().charFormat().fontItalic());
         m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(origCursor);
     }
     else {
         if (cursor.hasSelection()) {
@@ -531,7 +545,7 @@ void SvgTextEditor::setTextItalic(QFont::Style style)
 
 void SvgTextEditor::setTextDecoration(KoSvgText::TextDecoration decor)
 {
-    QTextCursor cursor = m_textEditorWidget.svgTextEdit->textCursor();
+    QTextCursor cursor = setTextSelection();
     QTextCharFormat currentFormat = m_textEditorWidget.richTextEdit->textCursor().charFormat();
     QTextCharFormat format;
     QString textDecoration = "inherit";
@@ -580,6 +594,7 @@ void SvgTextEditor::setTextDecoration(KoSvgText::TextDecoration decor)
             cursor.insertText(selectionModified);
         }
     }
+    m_textEditorWidget.richTextEdit->setTextCursor(cursor);
 }
 
 void SvgTextEditor::setTextUnderline()
@@ -621,18 +636,20 @@ void SvgTextEditor::setTextSuperScript()
 
 void SvgTextEditor::increaseTextSize()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextCharFormat format;
     int pointSize = m_textEditorWidget.richTextEdit->textCursor().charFormat().font().pointSize();
     if (pointSize<0) {
         pointSize = m_textEditorWidget.richTextEdit->textCursor().charFormat().font().pixelSize();
     }
-    qDebug()<<pointSize;
     format.setFontPointSize(pointSize+1.0);
     m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::decreaseTextSize()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextCharFormat format;
     int pointSize = m_textEditorWidget.richTextEdit->textCursor().charFormat().font().pointSize();
     if (pointSize<1) {
@@ -640,42 +657,71 @@ void SvgTextEditor::decreaseTextSize()
     }
     format.setFontPointSize(qMax(pointSize-1.0, 1.0));
     m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::setLineHeight(double lineHeightPercentage)
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextBlockFormat format = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
     format.setLineHeight(lineHeightPercentage, QTextBlockFormat::ProportionalHeight);
-     m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
+void SvgTextEditor::setLetterSpacing(double letterSpacing)
+{
+    QTextCursor cursor = setTextSelection();
+    if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
+        QTextCharFormat format;
+        format.setFontLetterSpacingType(QFont::AbsoluteSpacing);
+        format.setFontLetterSpacing(letterSpacing);
+        m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(cursor);
+    }
+    else {
+        if (cursor.hasSelection()) {
+            QString selectionModified = "<tspan style=\"letter-spacing:" + QString::number(letterSpacing) + "\">" + cursor.selectedText() + "</tspan>";
+            cursor.removeSelectedText();
+            cursor.insertText(selectionModified);
+        }
+    }
+}
 
 void SvgTextEditor::alignLeft()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextBlockFormat format = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
     format.setAlignment(Qt::AlignLeft);
     m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::alignRight()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextBlockFormat format = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
     format.setAlignment(Qt::AlignRight);
     m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::alignCenter()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextBlockFormat format = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
     format.setAlignment(Qt::AlignCenter);
     m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::alignJustified()
 {
+    QTextCursor oldCursor = setTextSelection();
     QTextBlockFormat format = m_textEditorWidget.richTextEdit->textCursor().blockFormat();
     format.setAlignment(Qt::AlignJustify);
     m_textEditorWidget.richTextEdit->textCursor().mergeBlockFormat(format);
+    m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
 }
 
 void SvgTextEditor::setSettings()
@@ -741,7 +787,6 @@ void SvgTextEditor::setSettings()
 
     settingsDialog.setButtons(KoDialog::Ok | KoDialog::Cancel);
     if (settingsDialog.exec() == QDialog::Accepted) {
-        qDebug() << "saving settings";
         // save  and set the settings
         QStringList writingSystems;
         for (int i = 0; i < writingSystemsModel->rowCount(); i++) {
@@ -750,9 +795,7 @@ void SvgTextEditor::setSettings()
                 writingSystems.append(QString::number(item->data().toInt()));
             }
         }
-        if (!writingSystems.isEmpty()) {
-            cfg.writeEntry("selectedWritingSystems", writingSystems.join(','));
-        }
+        cfg.writeEntry("selectedWritingSystems", writingSystems.join(','));
 
         if (textSettings.radioRichText->isChecked()) {
             cfg.writeEntry("EditorMode", (int)Richtext);
@@ -793,17 +836,17 @@ void SvgTextEditor::setSettings()
 
 void SvgTextEditor::slotToolbarToggled(bool)
 {
-
 }
 
-
 void SvgTextEditor::setFontColor(const KoColor &c)
 {
     QColor color = c.toQColor();
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
+        QTextCursor oldCursor = setTextSelection();
         QTextCharFormat format;
         format.setForeground(QBrush(color));
         m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
     }
     else {
         QTextCursor cursor = m_textEditorWidget.svgTextEdit->textCursor();
@@ -856,7 +899,9 @@ void SvgTextEditor::setFont(const QString &fontName)
     //This disables the style being set from the font-comboboxes too, so we need to rethink how we use that.
     format.setFontFamily(font.family());
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
+        QTextCursor oldCursor = setTextSelection();
         m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
     } else {
         QTextCursor cursor = m_textEditorWidget.svgTextEdit->textCursor();
         if (cursor.hasSelection()) {
@@ -870,9 +915,11 @@ void SvgTextEditor::setFont(const QString &fontName)
 void SvgTextEditor::setFontSize(qreal fontSize)
 {
     if (m_textEditorWidget.textTab->currentIndex() == Richtext) {
+        QTextCursor oldCursor = setTextSelection();
         QTextCharFormat format;
         format.setFontPointSize(fontSize);
         m_textEditorWidget.richTextEdit->mergeCurrentCharFormat(format);
+        m_textEditorWidget.richTextEdit->setTextCursor(oldCursor);
     } else {
         QTextCursor cursor = m_textEditorWidget.svgTextEdit->textCursor();
         if (cursor.hasSelection()) {
@@ -904,6 +951,15 @@ void SvgTextEditor::wheelEvent(QWheelEvent *event)
     }
 }
 
+QTextCursor SvgTextEditor::setTextSelection()
+{
+    QTextCursor orignalCursor(m_textEditorWidget.richTextEdit->textCursor());
+    if (!orignalCursor.hasSelection()){
+        m_textEditorWidget.richTextEdit->selectAll();
+    }
+    return orignalCursor;
+}
+
 void SvgTextEditor::applySettings()
 {
     KConfigGroup cfg(KSharedConfig::openConfig(), "SvgTextTool");
@@ -934,6 +990,10 @@ void SvgTextEditor::applySettings()
 
     QColor background = cfg.readEntry("colorEditorBackground", qApp->palette().background().color());
     palette.setBrush(QPalette::Active, QPalette::Background, QBrush(background));
+    m_textEditorWidget.richTextEdit->setStyleSheet(QString("background-color:%1").arg(background.name()));
+    m_textEditorWidget.svgStylesEdit->setStyleSheet(QString("background-color:%1").arg(background.name()));
+    m_textEditorWidget.svgTextEdit->setStyleSheet(QString("background-color:%1").arg(background.name()));
+
     QColor foreground = cfg.readEntry("colorEditorForeground", qApp->palette().text().color());
     palette.setBrush(QPalette::Active, QPalette::Text, QBrush(foreground));
 
@@ -945,6 +1005,8 @@ void SvgTextEditor::applySettings()
     }
     qobject_cast<KisFontComboBoxes*>(qobject_cast<QWidgetAction*>(actionCollection()->action("svg_font"))->defaultWidget())->refillComboBox(writingSystems);
 
+
+
     m_page->setUpdatesEnabled(true);
 }
 
@@ -967,7 +1029,7 @@ void SvgTextEditor::createActions()
 
     // File: new, open, save, save as, close
     KStandardAction::save(this, SLOT(save()), actionCollection());
-    KStandardAction::close(this, SLOT(close()), actionCollection());
+    KStandardAction::close(this, SLOT(slotCloseEditor()), actionCollection());
 
     // Edit
     KStandardAction::undo(this, SLOT(undo()), actionCollection());
@@ -983,8 +1045,9 @@ void SvgTextEditor::createActions()
     KStandardAction::replace(this, SLOT(replace()), actionCollection());
 
     // View
-    KStandardAction::zoomOut(this, SLOT(zoomOut()), actionCollection());
-    KStandardAction::zoomIn(this, SLOT(zoomIn()), actionCollection());
+    // WISH: we cannot zoom-in/out in rech-text mode
+    m_svgTextActions << KStandardAction::zoomOut(this, SLOT(zoomOut()), actionCollection());
+    m_svgTextActions << KStandardAction::zoomIn(this, SLOT(zoomIn()), actionCollection());
 #ifndef Q_OS_WIN
     // Insert:
     QAction * insertAction = createAction("svg_insert_special_character",
@@ -1077,9 +1140,19 @@ void SvgTextEditor::createActions()
     actionRegistry->propertizeAction("svg_background_color", bgColor);
     m_richTextActions << bgColor;
 
+    QWidgetAction *colorPickerAction = new QWidgetAction(this);
+    colorPickerAction->setToolTip(i18n("Pick a Color"));
+    KisScreenColorPicker *colorPicker = new KisScreenColorPicker(false);
+    connect(colorPicker, SIGNAL(sigNewColorPicked(KoColor)), fgColor, SLOT(setCurrentColor(KoColor)));
+    connect(colorPicker, SIGNAL(sigNewColorPicked(KoColor)), SLOT(setFontColor(KoColor)));
+    colorPickerAction->setDefaultWidget(colorPicker);
+    actionCollection()->addAction("svg_pick_color", colorPickerAction);
+    m_richTextActions << colorPickerAction;
+    actionRegistry->propertizeAction("svg_pick_color", colorPickerAction);
+
     QWidgetAction *lineHeight = new QWidgetAction(this);
-    lineHeight->setToolTip(i18n("Line height"));
     QDoubleSpinBox *spnLineHeight = new QDoubleSpinBox();
+    spnLineHeight->setToolTip(i18n("Line height"));
     spnLineHeight->setRange(0.0, 1000.0);
     spnLineHeight->setSingleStep(10.0);
     spnLineHeight->setSuffix("%");
@@ -1088,6 +1161,17 @@ void SvgTextEditor::createActions()
     actionCollection()->addAction("svg_line_height", lineHeight);
     m_richTextActions << lineHeight;
     actionRegistry->propertizeAction("svg_line_height", lineHeight);
+
+    QWidgetAction *letterSpacing = new QWidgetAction(this);
+    QDoubleSpinBox *spnletterSpacing = new QDoubleSpinBox();
+    spnletterSpacing->setToolTip(i18n("Letter Spacing"));
+    spnletterSpacing->setRange(-20.0, 20.0);
+    spnletterSpacing->setSingleStep(0.5);
+    connect(spnletterSpacing, SIGNAL(valueChanged(double)), SLOT(setLetterSpacing(double)));
+    letterSpacing->setDefaultWidget(spnletterSpacing);
+    actionCollection()->addAction("svg_letter_spacing", letterSpacing);
+    m_richTextActions << letterSpacing;
+    actionRegistry->propertizeAction("svg_letter_spacing", letterSpacing);
 }
 
 void SvgTextEditor::enableRichTextActions(bool enable)
@@ -1096,3 +1180,16 @@ void SvgTextEditor::enableRichTextActions(bool enable)
         action->setEnabled(enable);
     }
 }
+
+void SvgTextEditor::enableSvgTextActions(bool enable)
+{
+    Q_FOREACH(QAction *action, m_svgTextActions) {
+        action->setEnabled(enable);
+    }
+}
+
+void SvgTextEditor::slotCloseEditor()
+{
+    close();
+    emit textEditorClosed();
+}
